Attribute VB_Name = "Window_Pos"
Option Explicit

'==================================================================
'
' Window Position via API's
'
' 30 September 2001
'
' You're free in using this software, as long as the
' orignal copyright/author information remains intact.
' In other words, respecting the person(s) who made it
' possible for you to view/learn something of this software.
'
' Copyright  2001 - Diederik van der Boor
' http://www.codingdomain.com
'
'==================================================================



'------------------------------------------------------------------
' API's

Public Type RECT
  Left As Long
  Top As Long
  Right As Long
  Bottom As Long
End Type

Private Enum API_SetWindowPos_hWnd
  HWND_NOTOPMOST = -2
  HWND_TOPMOST = -1
  HWND_DESKTOP = 0
  HWND_BOTTOM = 1
  HWND_BROADCAST = &HFFFF&
End Enum

Private Enum API_SetWindowPos_Flags
  SWP_FRAMECHANGED = &H20
  SWP_HIDEWINDOW = &H80
  SWP_NOACTIVATE = &H10
  SWP_NOCOPYBITS = &H100
  SWP_NOMOVE = &H2
  SWP_NOOWNERZORDER = &H200
  SWP_NOREDRAW = &H8
  SWP_NOSIZE = &H1
  SWP_NOZORDER = &H4
  SWP_SHOWWINDOW = &H40
End Enum

Private Declare Function GetClientRect Lib "user32" (ByVal hWnd As Long, lpRect As RECT) As Long
Private Declare Function GetWindowRect Lib "user32" (ByVal hWnd As Long, lpRect As RECT) As Long
Private Declare Function SetWindowPos Lib "user32" (ByVal hWnd As Long, ByVal hWndInsertAfter As API_SetWindowPos_hWnd, ByVal X As Long, ByVal Y As Long, ByVal cx As Long, ByVal cy As Long, ByVal wFlags As API_SetWindowPos_Flags) As Long
Private Declare Function IsWindow Lib "user32" (ByVal hWnd As Long) As Long



'------------------------------------------------------------------
' Move the window

Public Sub WindowMove(hWnd As OLE_HANDLE, X As Long, Y As Long)
  If IsWindow(hWnd) = 0 Then Error 5
  If SetWindowPos(hWnd, 0, X, Y, 0, 0, (SWP_NOACTIVATE + SWP_NOSIZE)) = 0 Then _
    Err.Raise 12345, "WindowMove", "Could not move window!"
End Sub


'------------------------------------------------------------------
' Resize the window

Public Sub WindowResize(hWnd As OLE_HANDLE, Width As Long, Height As Long)
  If IsWindow(hWnd) = 0 Then Error 5
  If SetWindowPos(hWnd, 0, 0, 0, Width, Height, (SWP_NOACTIVATE + SWP_NOMOVE)) = 0 Then _
    Err.Raise 12345, "WindowResize", "Could not resize window!"
End Sub



'------------------------------------------------------------------
' Set always on top

Public Property Let WindowAlwaysOnTop(hWnd As OLE_HANDLE, OnTop As Boolean)
  If IsWindow(hWnd) = 0 Then Error 5
  If SetWindowPos(hWnd, IIf(OnTop, HWND_TOPMOST, HWND_NOTOPMOST), 0, 0, 0, 0, (SWP_NOSIZE Or SWP_NOMOVE Or SWP_NOACTIVATE)) = 0 Then _
    Err.Raise 12345, "WindowAlwaysOnTop", "Could not set always on top property!"
End Property


'------------------------------------------------------------------
' Get the entire position

Public Function WindowRect(hWnd As OLE_HANDLE, Optional AbsolutePos As Boolean) As RECT
  Dim rctBuff As RECT
  Dim Success As Boolean
  If IsWindow(hWnd) = 0 Then Error 5
  If AbsolutePos Then
    Success = GetWindowRect(hWnd, rctBuff)
  Else
    Success = GetClientRect(hWnd, rctBuff)
  End If
  If Not Success Then
    Err.Raise 12345, "WindowRect", "Could not get window pos/size"
  Else
    WindowRect = rctBuff
  End If
End Function

