Attribute VB_Name = "Window_Style"
Option Explicit

'==================================================================
'
' Window Styles
'
' 6 October 2001
'
' You're free in using this software, as long as the
' orignal copyright/author information remains intact.
' In other words, respecting the person(s) who made it
' possible for you to view/learn something of this software.
'
' Copyright  2001 - Diederik van der Boor
' http://www.codingdomain.com
'
'==================================================================



'------------------------------------------------------------------
' Constants

Public Enum WindowStyles
  ' Window Styles
  WS_BORDER = &H800000        ' Window has a thin-line border.
  WS_CAPTION = &HC00000       ' WS_BORDER Or WS_DLGFRAME (Has a caption)
  WS_CHILD = &H40000000       ' Is child window. cannot be used with WS_POPUP.
  WS_CLIPCHILDREN = &H2000000 ' Excludes the area occupied by child windows when drawing occurs within the parent window. This style is used when creating the parent window.
  WS_CLIPSIBLINGS = &H4000000 ' Clips child windows relative to each other; that is, when a particular child window receives a WM_PAINT message, the WS_CLIPSIBLINGS style clips all other overlapping child windows out of the region of the child window to be updated. If WS_CLIPSIBLINGS is not specified and child windows overlap, it is possible, when drawing within the client area of a child window, to draw within the client area of a neighboring child window.
  WS_DISABLED = &H8000000     ' Disables the window
  WS_DLGFRAME = &H400000      ' Window has a border of a style typically used with dialog boxes. When used, it cannot have a title bar.
  WS_GROUP = &H20000          ' Specifies the first control of a group of controls. The group consists of this first control and all controls defined after it, up to the next control with the WS_GROUP style. The first control in each group usually has the WS_TABSTOP style so that the user can move from group to group. The user can subsequently change the keyboard focus from one control in the group to the next control in the group by using the direction keys.
  WS_HSCROLL = &H100000       ' Horizontal Scrollbar
  WS_MAXIMIZE = &H1000000     ' Window is maximized
  WS_MAXIMIZEBOX = &H10000    ' Window has a Maximize button. Requires WS_SYSMENU, Cannot be combined with WS_EX_CONTEXTHELP
  WS_MINIMIZE = &H20000000    ' Window is minimized
  WS_MINIMIZEBOX = &H20000    ' Window has a Minimize button. Requires WS_SYSMENU, Cannot be combined with WS_EX_CONTEXTHELP
  WS_OVERLAPPED = &H0&        ' Creates an overlapped window. An overlapped window has a title bar and a border.
  WS_POPUP = &H80000000       ' Creates a pop-up window. cannot be used with WS_CHILD
  WS_SYSMENU = &H80000        ' Window has a window menu on its title bar, requires WS_CAPTION
  WS_TABSTOP = &H10000        ' Control can receive the keyboard focus when the user presses the tab key. Pressing the tab key changes the keyboard focus to the next control with the WS_TABSTOP style.
  WS_THICKFRAME = &H40000     ' Creates a window that has a sizing border
  WS_VISIBLE = &H10000000     ' Window is visible.
  WS_VSCROLL = &H200000       ' Vertical Scrollbar
  
  WS_OVERLAPPEDWINDOW = (WS_OVERLAPPED Or WS_CAPTION Or WS_SYSMENU Or WS_THICKFRAME Or WS_MINIMIZEBOX Or WS_MAXIMIZEBOX)
  WS_POPUPWINDOW = (WS_POPUP Or WS_BORDER Or WS_SYSMENU) ' Popup, Requires WS_CAPTION
  
'  WS_CHILDWINDOW = WS_CHILD
'  WS_ICONIC = WS_MINIMIZE
'  WS_SIZEBOX = WS_THICKFRAME
'  WS_TILED = WS_OVERLAPPED
'  WS_TILEDWINDOW = WS_OVERLAPPEDWINDOW
End Enum

Public Enum WindowStylesExtended
  WS_EX_ACCEPTFILES = &H10& ' Specifies that a window created with this style accepts drag-drop files.
  WS_EX_APPWINDOW = &H40000 ' Forces a top-level window onto the taskbar when the window is visible.
  WS_EX_CLIENTEDGE = &H200& ' Specifies that a window has a border with a sunken edge.
  WS_EX_CONTEXTHELP = &H400 ' Includes a question mark in the title bar of the window. When the user clicks the question mark, the cursor changes to a question mark with a pointer. If the user then clicks a child window, the child receives a WM_HELP message. The child window should pass the message to the parent window procedure, which should call the WinHelp function using the HELP_WM_HELP command. The Help application displays a pop-up window that typically contains help for the child window. WS_EX_CONTEXTHELP cannot be used with the WS_MAXIMIZEBOX or WS_MINIMIZEBOX styles.
  
  WS_EX_CONTROLPARENT = &H10000 ' Allows the user to navigate among the child windows of the window by using the tab key.
  WS_EX_DLGMODALFRAME = &H1 ' Creates a window that has a double border; the window can, optionally, be created with a title bar by specifying the WS_CAPTION style in the dwStyle parameter.
  WS_EX_LEFT = &H0 ' Window has generic "left-aligned" properties. This is the default.
  WS_EX_LEFTSCROLLBAR = &H4000 ' If the shell language is Hebrew, Arabic, or another language that supports reading order alignment, the vertical scroll bar (if present) is to the left of the client area. For other languages, the style is ignored and not treated as an error.
  WS_EX_LTRREADING = &H0 ' The window text is displayed using Left to Right reading-order properties. This is the default.
  WS_EX_MDICHILD = &H40 ' Creates an MDI child window.
  WS_EX_NOPARENTNOTIFY = &H4& ' Specifies that a child window created with this style does not send the WM_PARENTNOTIFY message to its parent window when it is created or destroyed.
  WS_EX_RIGHT = &H1000 ' The window has generic "right-aligned" properties. This depends on the window class. This style has an effect only if the shell language is Hebrew, Arabic, or another language that supports reading order alignment; otherwise, the style is ignored and not treated as an error.
  WS_EX_RIGHTSCROLLBAR = &H0 ' Vertical scroll bar (if present) is to the right of the client area. This is the default.
  WS_EX_RTLREADING = &H2000 ' If the shell language is Hebrew, Arabic, or another language that supports reading order alignment, the window text is displayed using Right to Left reading-order properties. For other languages, the style is ignored and not treated as an error.
  WS_EX_STATICEDGE = &H20000 ' Creates a window with a three-dimensional border style intended to be used for items that do not accept user input.
  WS_EX_TOOLWINDOW = &H80 ' Creates a tool window; that is, a window intended to be used as a floating toolbar. A tool window has a title bar that is shorter than a normal title bar, and the window title is drawn using a smaller font. A tool window does not appear in the taskbar or in the dialog that appears when the user presses alt+tab. If a tool window has a system menu, its icon is not displayed on the title bar. However, you can display the system menu by right-clicking or by typing alt+space.
  WS_EX_TOPMOST = &H8 ' Specifies that a window created with this style should be placed above all non-topmost windows and should stay above them, even when the window is deactivated. To add or remove this style, use the SetWindowPos function.
  WS_EX_TRANSPARENT = &H20& ' Specifies that a window created with this style should not be painted until siblings beneath the window (that were created by the same thread) have been painted. The window appears transparent because the bits of underlying sibling windows have already been painted. To achieve transparency without these restrictions, use the SetWindowRgn function.
  WS_EX_WINDOWEDGE = &H100 ' Specifies that a window has a border with a raised edge.
  WS_EX_OVERLAPPEDWINDOW = (WS_EX_WINDOWEDGE Or WS_EX_CLIENTEDGE) ' Combines the WS_EX_CLIENTEDGE and WS_EX_WINDOWEDGE styles.
  WS_EX_PALETTEWINDOW = (WS_EX_WINDOWEDGE Or WS_EX_TOOLWINDOW Or WS_EX_TOPMOST) ' Combines the WS_EX_WINDOWEDGE, WS_EX_TOOLWINDOW, and WS_EX_TOPMOST styles.
End Enum


'------------------------------------------------------------------
' API's

' Window field offsets for GetWindowLong() and GetWindowWord()
Private Enum API_GetWindowLong
  GWL_STYLE = (-16)
  GWL_EXSTYLE = (-20)
'  GWL_WNDPROC = (-4)
'  GWL_HINSTANCE = (-6)
'  GWL_HWNDPARENT = (-8)
'  GWL_USERDATA = (-21)
'  GWL_ID = (-12)
End Enum

Private Enum API_SetWindowPos
  SWP_FRAMECHANGED = &H20
  SWP_NOACTIVATE = &H10
  SWP_NOMOVE = &H2
  SWP_NOSIZE = &H1
  SWP_NOZORDER = &H4
'  SWP_HIDEWINDOW = &H80
'  SWP_NOCOPYBITS = &H100
'  SWP_NOOWNERZORDER = &H200
'  SWP_NOREDRAW = &H8
'  SWP_SHOWWINDOW = &H40
'  SWP_DRAWFRAME = SWP_FRAMECHANGED
'  SWP_NOREPOSITION = SWP_NOOWNERZORDER
End Enum

Private Declare Function GetWindowLong Lib "user32" Alias "GetWindowLongA" (ByVal hWnd As Long, ByVal nIndex As API_GetWindowLong) As Long
Private Declare Function SetWindowLong Lib "user32" Alias "SetWindowLongA" (ByVal hWnd As Long, ByVal nIndex As API_GetWindowLong, ByVal dwNewLong As Long) As Long
Private Declare Function SetWindowPos Lib "user32" (ByVal hWnd As Long, ByVal hWndInsertAfter As Long, ByVal X As Long, ByVal Y As Long, ByVal cx As Long, ByVal cy As Long, ByVal wFlags As API_SetWindowPos) As Long
Private Declare Function IsWindow Lib "user32" (ByVal hWnd As Long) As Long


'------------------------------------------------------------------
' Assign or get a window style

Public Property Get WindowStyle(hWnd As OLE_HANDLE) As WindowStyles
  If IsWindow(hWnd) = 0 Then Error 5
  WindowStyle = GetWindowLong(hWnd, GWL_STYLE)
End Property

Public Property Let WindowStyle(hWnd As OLE_HANDLE, NewValue As WindowStyles)
  If IsWindow(hWnd) = 0 Then Error 5
  If SetWindowLong(hWnd, GWL_STYLE, NewValue) = 0 Then _
    Err.Raise 12345, "WindowStyle", "Could not change window style!"
End Property

'------------------------------------------------------------------
' Same with extended styles

Public Property Get WindowExStyle(hWnd As OLE_HANDLE) As WindowStylesExtended
  If IsWindow(hWnd) = 0 Then Error 5
  WindowExStyle = GetWindowLong(hWnd, GWL_EXSTYLE)
End Property

Public Property Let WindowExStyle(hWnd As OLE_HANDLE, NewValue As WindowStylesExtended)
  If IsWindow(hWnd) = 0 Then Error 5
  If SetWindowLong(hWnd, GWL_EXSTYLE, NewValue) = 0 Then _
    Err.Raise 12345, "WindowStyle", "Could not change window extended style!"
End Property


'------------------------------------------------------------------
' Updater for VB windows

Public Sub WindowUpdateStyle(hWnd As OLE_HANDLE)
  If IsWindow(hWnd) = 0 Then Error 5
  Call SetWindowPos(hWnd, 0, 0, 0, 0, 0, SWP_FRAMECHANGED Or SWP_NOACTIVATE Or SWP_NOMOVE Or SWP_NOSIZE Or SWP_NOZORDER)
End Sub
